"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _createFunction = _interopRequireDefault(require("@polkadot/api-metadata/storage/fromMetadata/createFunction"));

var _Storage = require("@polkadot/types/Metadata/v7/Storage");

var _types = require("@polkadot/types");

var _util = require("@polkadot/util");

var _utilCrypto = require("@polkadot/util-crypto");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const prefix = 'Block';
const section = 'Block'; // Small helper function to factorize code on this page.

const createMethod = (method, key, _ref) => {
  let {
    documentation,
    type
  } = _ref;
  const creator = (0, _createFunction.default)({
    meta: {
      documentation: new _types.Vec(_types.Text, documentation),
      modifier: (0, _types.createType)('StorageEntryModifierV7', 1),
      // default
      type: new _Storage.StorageEntryType(type, (0, _util.isString)(type) ? 0 : 1),
      toJSON: () => key
    },
    method,
    prefix,
    section
  }, {
    key,
    skipHashing: true
  });
  return function () {
    return (0, _utilCrypto.blake2AsU8a)(creator(...arguments));
  };
};

var _default = {
  bestHash: createMethod('bestHash', 'bst:hsh', {
    documentation: ['Best hash'],
    type: 'Hash'
  }),
  bestNumber: createMethod('bestNumber', 'bst:num', {
    documentation: ['Best block'],
    type: 'BlockNumber'
  }),
  blockByHash: createMethod('blockByHash', 'blk:hsh:', {
    documentation: ['Retrieve block by hash'],
    type: {
      key: 'Hash',
      value: 'Bytes'
    }
  }),
  hashByNumber: createMethod('hashByNumber', 'hsh:num:', {
    documentation: ['Retrieve hash by number'],
    type: {
      key: 'u256',
      value: 'Hash'
    }
  }),
  headerByHash: createMethod('headerByHash', 'hdr:hsh:', {
    documentation: ['Retrieve header by hash'],
    type: {
      key: 'Hash',
      value: 'Bytes'
    }
  })
};
exports.default = _default;