"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _util = require("@polkadot/util");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
class Cache {
  constructor(maxEntries) {
    this._cache = new Map();
    this._maxEntries = void 0;
    (0, _util.assert)(maxEntries >= 10, "Need a minimum of 10 entries, ".concat(maxEntries, " found"));
    this._maxEntries = maxEntries;
  }

  get(key) {
    const cached = this._cache.get(key);

    if (cached) {
      return cached.value;
    }

    return null;
  }

  set(key, value) {
    const cached = this._cache.get(key);

    if (cached) {
      cached.value = value;
      return;
    }

    if (this._cache.size > this._maxEntries) {
      this._cache.clear();
    }

    this._cache.set(key, {
      value
    });
  }

}

exports.default = Cache;