"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _path = _interopRequireDefault(require("path"));

var _Cache = _interopRequireDefault(require("./Cache"));

var _constants = require("./constants");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const CACHE_SIZES = {
  idx: 6 * 512,
  key: 4 * 512,
  val: 2 * 512
};

class Files {
  constructor(base, file, options) {
    this._isTrie = false;
    this._isOpen = false;
    this._fds = [];
    this._path = void 0;
    this._isTrie = options.isTrie;
    this._path = _path.default.join(base, _constants.DB_VERSION, file);

    _mkdirp.default.sync(this._path);
  }

  close() {
    this._fds.forEach(fds => Object.values(fds).forEach((_ref) => {
      let {
        fd
      } = _ref;
      return _fs.default.closeSync(fd);
    }));

    this._isOpen = false;
  }

  open() {
    for (let i = 0; i < _constants.DB_MAX_FILES; i++) {
      this._fds[i] = ['idx', 'key', 'val'].reduce((fds, type) => {
        const count = "0".concat(i.toString(16)).slice(-2);

        const file = _path.default.join(this._path, "".concat(count, ".").concat(type));

        if (!_fs.default.existsSync(file)) {
          _fs.default.writeFileSync(file, new Uint8Array(type === 'idx' ? _constants.HDR_TOTAL_SIZE : 0));
        }

        const cache = new _Cache.default(CACHE_SIZES[type]);

        const fd = _fs.default.openSync(file, 'r+');

        const size = _fs.default.fstatSync(fd).size;

        fds[type] = {
          cache,
          fd,
          file,
          size
        };
        return fds;
      }, {});
    }
  }

  __append(type, fileAt, buffer) {
    const fd = this._fds[fileAt][type];
    const at = fd.size;

    _fs.default.writeSync(fd.fd, buffer, 0, buffer.length, at);

    fd.size += buffer.length;
    fd.cache.set(at, buffer);
    return at;
  }

  __read(type, fileAt, at, length) {
    const fd = this._fds[fileAt][type];
    const cached = fd.cache.get(at);

    if (cached) {
      return cached;
    }

    const buffer = new Uint8Array(length);

    _fs.default.readSync(fd.fd, buffer, 0, length, at);

    fd.cache.set(at, buffer);
    return buffer;
  }

  __update(type, fileAt, at, buffer) {
    const fd = this._fds[fileAt][type];

    _fs.default.writeSync(fd.fd, buffer, 0, buffer.length, at);

    fd.cache.set(at, buffer);
    return at;
  }

  _appendHdr(fileAt, buffer) {
    return this.__append('idx', fileAt, buffer) / _constants.HDR_TOTAL_SIZE;
  }

  _appendKey(fileAt, buffer) {
    return this.__append('key', fileAt, buffer) / _constants.KEY_TOTAL_SIZE;
  }

  _appendVal(fileAt, buffer) {
    return this.__append('val', fileAt, buffer);
  }

  _readHdr(fileAt, at) {
    return this.__read('idx', fileAt, at * _constants.HDR_TOTAL_SIZE, _constants.HDR_TOTAL_SIZE);
  }

  _readKey(fileAt, at) {
    return this.__read('key', fileAt, at * _constants.KEY_TOTAL_SIZE, _constants.KEY_TOTAL_SIZE);
  }

  _readVal(fileAt, at, length) {
    return this.__read('val', fileAt, at, length);
  }

  _updateHdr(fileAt, at, buffer) {
    this.__update('idx', fileAt, at * _constants.HDR_TOTAL_SIZE, buffer);
  }

  _updateKey(fileAt, at, buffer) {
    this.__update('key', fileAt, at * _constants.KEY_TOTAL_SIZE, buffer);
  }

}

exports.default = Files;