"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Files = _interopRequireDefault(require("./Files"));

var _constants = require("./constants");

var _util = require("./util");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
// import { logger } from '@polkadot/util';
// const l = logger('db/struct');
class Impl extends _Files.default {
  _findValue(key) {
    let value = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    let withValue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    let keyIndex = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
    let hdrAt = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 0;

    const hdr = this._readHdr(key.fileAt, hdrAt);

    const parsedHdr = (0, _util.parseHdr)(hdr, key.nibbles[keyIndex]);

    if (parsedHdr.isKey) {
      return this.__retrieveKey(key, value, withValue, keyIndex, hdr, hdrAt, parsedHdr.linkTo);
    } else if (parsedHdr.linkTo) {
      return this._findValue(key, value, withValue, keyIndex + 1, parsedHdr.linkTo);
    }

    return this.__retrieveEmpty(key, value, keyIndex, hdr, hdrAt);
  }

  __appendNewValue(key, valData) {
    return {
      valAt: this._appendVal(key.fileAt, valData),
      valData,
      valSize: valData.length
    };
  }

  __appendNewKeyValue(key, value) {
    const kvInfo = this.__appendNewValue(key, value);

    kvInfo.keyData = (0, _util.newKey)(key, kvInfo);
    kvInfo.keyAt = this._appendKey(key.fileAt, kvInfo.keyData);
    return kvInfo;
  }

  __retrieveEmpty(key, value, keyIndex, hdr, hdrAt) {
    if (!value) {
      return null;
    }

    const hdrIndex = key.nibbles[keyIndex];

    const newInfo = this.__appendNewKeyValue(key, value);

    (0, _util.modifyHdr)(hdr, hdrIndex, newInfo.keyAt, true);

    this._updateHdr(key.fileAt, hdrAt, hdr);

    return newInfo;
  }

  __retrieveKey(key, value, withValue, keyIndex, hdr, hdrAt, keyAt) {
    const hdrIndex = key.nibbles[keyIndex];

    const keyData = this._readKey(key.fileAt, keyAt);

    const prevKey = (0, _util.serializeKey)(keyData.subarray(0, _constants.KEY_DATA_SIZE));
    let matchIndex = keyIndex; // see if this key matches fully with what we are supplied

    while (matchIndex < key.nibbles.length) {
      if (prevKey.nibbles[matchIndex] !== key.nibbles[matchIndex]) {
        break;
      }

      matchIndex++;
    } // we have a match, either retrieve or update


    if (matchIndex === key.nibbles.length) {
      if (value) {
        const {
          valAt,
          valData,
          valSize
        } = this.__appendNewValue(key, value);

        this._updateKey(key.fileAt, keyAt, (0, _util.modifyKey)(keyData, valAt, valSize));

        return {
          keyAt,
          keyData,
          valAt,
          valData,
          valSize
        };
      }

      const {
        valAt,
        valSize
      } = (0, _util.parseKey)(keyData);
      const valData = withValue ? this._readVal(key.fileAt, valAt, valSize) : null;
      return {
        keyAt,
        keyData,
        valAt,
        valData,
        valSize
      };
    } else if (!value) {
      return null;
    } // write the new key and create a header with the 2 new values


    const newKv = this.__appendNewKeyValue(key, value);

    let depth = matchIndex - keyIndex - 1; // write the last header - this contains the old and new keys at the correct indexes

    let lastAt = this._appendHdr(key.fileAt, (0, _util.newHdr)([{
      dataAt: keyAt,
      hdrIndex: prevKey.nibbles[matchIndex],
      isKey: true
    }, {
      dataAt: newKv.keyAt,
      hdrIndex: key.nibbles[matchIndex],
      isKey: true
    }])); // make a tree from the header we are modifying down to the others


    for (let offset = 1; depth > 0; depth--, offset++) {
      lastAt = this._appendHdr(key.fileAt, (0, _util.newHdr)([{
        dataAt: lastAt,
        hdrIndex: key.nibbles[matchIndex - offset],
        isKey: false
      }]));
    }

    this._updateHdr(key.fileAt, hdrAt, (0, _util.modifyHdr)(hdr, hdrIndex, lastAt, false));

    return newKv;
  }

}

exports.default = Impl;