"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.readU32 = readU32;
exports.writeU32 = writeU32;
exports.modifyHdr = modifyHdr;
exports.modifyKey = modifyKey;
exports.newHdr = newHdr;
exports.newKey = newKey;
exports.parseHdr = parseHdr;
exports.parseKey = parseKey;
exports.serializeKey = serializeKey;

var _constants = require("./constants");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
function readU32(u8a, offset) {
  // reverse the writing, highest bits goes first, lowest are last
  return (u8a[offset] << 24) + (u8a[offset + 1] << 16) + (u8a[offset + 2] << 8) + u8a[offset + 3];
}

function writeU32(u8a, value, offset) {
  // write the highest bits first - this way, when flagged, we only need to
  // read a single byte (this is useful in the cases where we re-encode a trie)
  u8a[offset] = value >> 24 & 0xff;
  u8a[offset + 1] = value >> 16 & 0xff;
  u8a[offset + 2] = value >> 8 & 0xff;
  u8a[offset + 3] = value & 0xff;
}

function modifyHdr(hdr, hdrIndex, linkAt, isKey) {
  writeU32(hdr, isKey ? linkAt | _constants.BITS_F : linkAt, hdrIndex * _constants.HDR_ENTRY_SIZE);
  return hdr;
}

function modifyKey(keyData, valAt, valSize) {
  writeU32(keyData, valAt, _constants.KEY_DATA_SIZE);
  writeU32(keyData, valSize, _constants.KEY_DATA_SIZE + _constants.U32_SIZE);
  return keyData;
}

function newHdr(indexes) {
  const hdr = new Uint8Array(_constants.HDR_TOTAL_SIZE);
  indexes.forEach((_ref) => {
    let {
      dataAt,
      hdrIndex,
      isKey
    } = _ref;
    modifyHdr(hdr, hdrIndex, dataAt, isKey);
  });
  return hdr;
}

function newKey(key, _ref2) {
  let {
    valAt,
    valSize
  } = _ref2;
  const keyData = new Uint8Array(_constants.KEY_TOTAL_SIZE);
  keyData.set(key.buffer, 0);
  return modifyKey(keyData, valAt, valSize);
}

function parseHdr(hdr, hdrIndex) {
  const u32 = readU32(hdr, hdrIndex * _constants.HDR_ENTRY_SIZE);
  const linkTo = u32 & _constants.BITS_U;
  return {
    isKey: linkTo !== u32,
    linkTo
  };
}

function parseKey(keyData) {
  return {
    valAt: readU32(keyData, _constants.KEY_DATA_SIZE),
    valSize: readU32(keyData, _constants.KEY_DATA_SIZE + _constants.U32_SIZE)
  };
}

function serializeKey(buffer) {
  const nibbles = new Uint8Array(_constants.KEY_PARTS_SIZE);
  let offset = 0;

  for (let i = 0; i < _constants.KEY_DATA_SIZE; i++) {
    const item = buffer[i];
    nibbles[offset] = item & 0b1111;
    nibbles[offset + 1] = item >> 4;
    offset += 2;
  }

  return {
    buffer,
    fileAt: buffer[0] % _constants.DB_MAX_FILES,
    nibbles
  };
}